//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_VEHICLE_H
#define PROTOCOL_VEHICLE_H

#include <thread>
#include "VehicleEndpoint.h"
#include "PlatoonService.h"
#include "NetworkingLib/Networking.h"
#include "NetworkingLib/ServiceServer.h"
#include "NetworkingLib/ServiceClient.h"
#include "NetworkingLib/Resolver.h"
#include "NetworkingLib/TimedValue.h"

namespace platoonProtocol
{
class Vehicle : public std::enable_shared_from_this<Vehicle>
{
public:
    using Ptr = std::shared_ptr<Vehicle>;
    using Callback = std::function<void()>;
    using VehicleEndpoints = std::vector<VehicleEndpoint>;

    enum class Role
    {
        LEADER, FOLLOWER
    };

    virtual ~Vehicle() = default;

    bool isIdle();

    bool isPlatoonRunning();

    void createPlatoon();

    void leavePlatoon();

    virtual void stop();

    void addVehicleEndpoints(const VehicleEndpoints & endpoints);

    VehicleEndpoint getOwnEndpoint() const
    { return ownEndpoint; }

    Role getRole() const noexcept
    { return role; }

    void setOnRunningPlatoonCallback(const Callback & callback)
    { onRunningPlatoonCallback = callback; }

    void setOnLeavingPlatoonCallback(const Callback & callback)
    { onLeavingPlatoonCallback = callback; }

protected:
    Vehicle(
        networking::Networking & net,
        VehicleId ownVehicleId,
        Role role);

    enum class State
    {
        IDLE,
        CREATING_PLATOON,
        RUNNING_PLATOON
    };

    static const PlatoonConfig DEFAULT_PLATOON_CONFIG;

    const VehicleEndpoint ownEndpoint;
    PlatoonId platoonId;
    networking::time::TimedAtomicValue<PlatoonConfig> platoonConfig{DEFAULT_PLATOON_CONFIG};
    State state{State::IDLE};
    std::mutex stateMutex;
    networking::Networking & net;
    networking::service::Server<PlatoonService>::Ptr platoonCreateServiceServer;
    networking::service::Client<PlatoonService>::Ptr platoonCreateServiceClient;
    std::vector<VehicleEndpoint> vehicleEndpoints;
    std::vector<VehicleEndpoint>::iterator vehicleEndpointsIter;
    Callback onRunningPlatoonCallback;
    Callback onLeavingPlatoonCallback;

    virtual void doCreatePlatoon();

    virtual void doRunPlatoon() = 0;

    virtual void doLeavePlatoon() = 0;

    virtual void makePlatoonCreateRequest(const VehicleEndpoint & vehicleEndpoint) = 0;

    virtual void makePlatoonCreateResponse(const networking::Resolver::Endpoint & clientEndpoint,
                                           const PlatoonMessage & request,
                                           PlatoonMessage & response) = 0;

    void makePlatoonCreateRequestToNextEndpoint();

    void setState(State newState);

private:
    Role role;
};
}


#endif //PROTOCOL_VEHICLE_H
