//
// Created by philipp on 29.11.17.
//

#ifndef PROTOCOL_ENPOINTINFO_H
#define PROTOCOL_ENPOINTINFO_H

#include <string>
#include <boost/asio/ip/tcp.hpp>
#include "Protocol.h"

using boost::asio::ip::tcp;

namespace platoonProtocol
{

class VehicleEndpoint
{
public:
    friend bool operator==(const VehicleEndpoint & lhs, const VehicleEndpoint & rhs);

    VehicleEndpoint() = default;

    VehicleEndpoint(VehicleId vehicleId, const std::string & host)
        : vehicleId(vehicleId)
          , host(host)
          , tcpPort(TCP_PORT)
          , udpPort(UDP_PORT)
    {
#ifdef PLATOON_PROTOCOL_ENABLE_LOCAL_TESTING
        // To test this locally, we must ensure that each vehicle has a different port.
        tcpPort += vehicleId;
        udpPort += vehicleId;
#endif
    }

    VehicleId getVehicleId() const noexcept
    { return vehicleId; }

    std::string getHost() const noexcept
    { return host; }

    std::uint16_t getTcpPort() const noexcept
    { return tcpPort; }

    std::uint16_t getUdpPort() const noexcept
    { return udpPort; }

    explicit operator bool() const noexcept
    { return vehicleId != 0; }

private:
    VehicleId vehicleId{0};
    std::string host{""};
    std::uint16_t tcpPort{0};
    std::uint16_t udpPort{0};
};

bool operator==(const VehicleEndpoint & lhs, const VehicleEndpoint & rhs);

}

#endif //PROTOCOL_ENPOINTINFO_H
